use strict;

sub mkprint
{
    my $line = shift;
    print MAKE $line."\n";
}
sub mkprinttab
{
    my $line = shift;
    print MAKE "\t".$line."\n";
}
sub askval
{
    my $value = shift;
    my $title = shift;
    my $val;
    print "$title [$$value]: ";
    $val = <STDIN>;
    chomp($val);
    if ($val)
    {
	$$value = $val;
    }
}

my $mnvers = eval('use MN; return $MN::VERSION');
if (!$mnvers || $mnvers < 1.0)
{
    print "\n";
    if ($mnvers)
    {
	print "*** Error: MyNews requires at least version 1.0 of MN perl module (found $mnvers) ***\n";
    }
    else
    {
	print "*** Error: MN perl module not found ***\n";
    }
    print "\n";
    exit;
}

my $install_path = '/usr';
my $conf_path = '/etc';
my $dbtype = 'pgsql';
my $dbname = 'mynews';
my $dbuser = 'mynews';
my $dbpass = '';
my $webdir = '/var/www/mynews';
my $weburl = 'http://localhost';
my $mnurl = 'http://www.neosystem.com/mynews';
my $mnmail = 'MyNews <mynews@localhost>';
my $mnuser = 'mynews';
my $mngroup = 'apache';

askval(\$install_path, "Install dir");
askval(\$conf_path, "Conf dir");
if (open(CONF, "$conf_path/mynews.conf"))
{
    my $line;
    my $key;
    my $val;
    while ($line = <CONF>)
    {
	if ($line =~ /^\s*(\S+)\s*=\s*(.*)$/)
	{
	    $key = $1;
	    $val = $2;
	    if ($key eq 'type')
	    {
		$dbtype = $val;
	    }
	    elsif ($key eq 'database')
	    {
		$dbname = $val;
	    }
	    elsif ($key eq 'user')
	    {
		$dbuser = $val;
	    }
	    elsif ($key eq 'pass')
	    {
		$dbpass = $val;
	    }
	    elsif ($key eq 'webdir')
	    {
		$webdir = $val;
	    }
	    elsif ($key eq 'weburl')
	    {
		$weburl = $val;
	    }
	    elsif ($key eq 'mnurl')
	    {
		$mnurl = $val;
	    }
	    elsif ($key eq 'mnmail')
	    {
		$mnmail = $val;
	    }
	}
    }
    close(CONF);
    my @st = stat("$conf_path/mynews.conf");
    my $mnu = getpwuid($st[4]);
    my $mng = getgrgid($st[5]);
    if ($mnu)
    {
	$mnuser = $mnu;
    }
    if ($mng)
    {
	$mngroup = $mng;
    }
}
else
{
    askval(\$dbtype, "Database type (pgsql|mysql)");
    askval(\$dbname, "Database name");
    askval(\$dbuser, "Database user");
    askval(\$dbpass, "Database pass");
    askval(\$webdir, "MyNews local site directory");
    askval(\$weburl, "MyNews local site URL");
    askval(\$mnurl, "MyNews distant site URL");
    askval(\$mnmail, "MyNews email address");
    my $again = 1;
    my $confirm;
    my @us;
    my $ugid;
    while ($again)
    {
	$again = 0;
	askval(\$mnuser, "MyNews unix user");
	@us = getpwnam($mnuser);
	if (!defined(@us))
	{
	    print "User $mnuser unknown\n";
	    $again = 1;
	    next;
	}
	elsif ($us[2] == 0)
	{
	    print "Warning: It is not recommended to run mynews as superuser!\n";
	    $confirm = 'n';
	    askval(\$confirm, "Confirm? (y/n)");
	    if ($confirm ne 'y')
	    {
		$again = 1;
	    }
	}
	$ugid = $us[3];
	undef @us;
    }
    $again = 1;
    my $gid;
    while ($again)
    {
	$again = 0;
	askval(\$mngroup, "MyNews unix group");
	$gid = getgrnam($mngroup);
	if (!defined($gid))
	{
	    print "Group $mngroup unknown\n";
	    $again = 1;
	}
	elsif ($gid != $ugid)
	{
	    print "Warning: It seems that user $mnuser doesn't belong to $mngroup group\n";
	    $confirm = 'n';
	    askval(\$confirm, "Confirm? (y/n)");
	    if ($confirm ne 'y')
	    {
		$again = 1;
	    }
	}
    undef $gid;
    }
}
print "Writing Makefile for mynews\n";
open(MAKE, ">Makefile");

mkprint('# Makefile to install mynews script');
mkprint();
mkprint('INSTALL_PATH = '.$install_path);
mkprint('CONF_PATH = '.$conf_path);
mkprint('DB_TYPE = '.$dbtype);
mkprint('DB_NAME = '.$dbname);
mkprint('DB_USER = '.$dbuser);
mkprint('DB_PASS = '.$dbpass);
mkprint('WEB_DIR = '.$webdir);
mkprint('WEB_URL = '.$weburl);
mkprint('MN_URL = '.$mnurl);
mkprint('MN_MAIL = '.$mnmail);
mkprint('MN_USER = '.$mnuser);
mkprint('MN_GROUP = '.$mngroup);
mkprint();
mkprint('BUILD_DIR = build');
mkprint('MAKEFILE = Makefile');
mkprint('MAKEFILE_OLD = Makefile.old');
mkprint('MYNEWS = mynews');
mkprint('MYNEWS_CONF = mynews.conf');
mkprint('MYDOC = mydoc');
mkprint('MYCHECK = mycheck');
mkprint();
mkprint('PERL = '.$^X);
mkprint('SHELL = /bin/sh');
mkprint('NOOP = $(SHELL) -c true');
mkprint('NOECHO = @');
mkprint('ECHO = echo');
mkprint('ECHON = echo -n');
mkprint('CHMOD = chmod');
mkprint('CHOWN = chown');
mkprint('CHGRP = chgrp');
mkprint('CP = cp');
mkprint('MV = mv');
mkprint('MKDIR = mkdir -p');
mkprint('RM_F = rm -f');
mkprint('RM_RF = rm -rf');
mkprint('PRINT_CONF = echo >> $(BUILD_DIR)/$(MYNEWS_CONF)');
mkprint();
mkprint('all : $(BUILD_DIR) $(BUILD_DIR)/$(MYNEWS_CONF)');
mkprinttab('$(NOECHO) $(ECHO)');
mkprinttab('$(NOECHO) $(ECHO) MyNews built successfully');
mkprinttab('$(NOECHO) $(ECHO)');
mkprint();
mkprint('clean :');
mkprinttab('$(RM_RF) $(BUILD_DIR)');
mkprinttab('$(MV) $(MAKEFILE) $(MAKEFILE_OLD)');
mkprint();
mkprint('realclean : clean');
mkprinttab('$(RM_F) $(MAKEFILE_OLD)');
mkprint();
mkprint('$(BUILD_DIR) :');
mkprinttab('$(MKDIR) $(BUILD_DIR)');
mkprint();
mkprint('$(BUILD_DIR)/$(MYNEWS_CONF) :');
mkprinttab('$(PRINT_CONF) "# MyNews configuration file"');
mkprinttab('$(PRINT_CONF) "backend default"');
mkprinttab('$(PRINT_CONF) "	type = $(DB_TYPE)"');
mkprinttab('$(PRINT_CONF) "	database = $(DB_NAME)"');
mkprinttab('$(PRINT_CONF) "	user = $(DB_USER)"');
mkprinttab('$(PRINT_CONF) "	pass = $(DB_PASS)"');
mkprinttab('$(PRINT_CONF) "end backend"');
mkprinttab('$(PRINT_CONF) "webdir = $(WEB_DIR)"');
mkprinttab('$(PRINT_CONF) "weburl = $(WEB_URL)"');
mkprinttab('$(PRINT_CONF) "mnurl = $(MN_URL)"');
mkprinttab('$(PRINT_CONF) "mnmail = $(MN_MAIL)"');
mkprint();
mkprint('dbinit :');
mkprinttab('$(PERL) $(MYNEWS) -c $(BUILD_DIR)/$(MYNEWS_CONF) -ca dbinit db/mynews.$(DB_TYPE)');
mkprinttab('$(PERL) $(MYNEWS) -c $(BUILD_DIR)/$(MYNEWS_CONF) -ca admininit');
mkprint();
mkprint('dbreset :');
mkprinttab('$(PERL) $(MYNEWS) -c $(BUILD_DIR)/$(MYNEWS_CONF) dbinit db/drop.$(DB_TYPE)');
mkprint();
mkprint('install : all dbinit install_scripts install_conf');
mkprinttab('$(NOECHO) $(ECHO)');
mkprinttab('$(NOECHO) $(ECHO) MyNews installed successfully');
mkprinttab('$(NOECHO) $(ECHO)');
mkprint();
mkprint('install_scripts :');
mkprinttab('$(CP) $(MYNEWS) $(INSTALL_PATH)/bin');
mkprinttab('$(CHMOD) 0755 $(INSTALL_PATH)/bin/$(MYNEWS)');
mkprinttab('$(CP) $(MYDOC) $(INSTALL_PATH)/bin');
mkprinttab('$(CHMOD) 0755 $(INSTALL_PATH)/bin/$(MYDOC)');
mkprinttab('$(CP) $(MYCHECK) $(INSTALL_PATH)/bin');
mkprinttab('$(CHMOD) 0755 $(INSTALL_PATH)/bin/$(MYCHECK)');
mkprint();
mkprint('install_conf : $(CONF_PATH)/$(MYNEWS_CONF)');
mkprinttab('$(NOECHO) $(NOOP)');
mkprint();
mkprint('$(CONF_PATH)/$(MYNEWS_CONF) :');
mkprinttab('$(CP) $(BUILD_DIR)/$(MYNEWS_CONF) $(CONF_PATH)');
mkprinttab('$(CHMOD) 0640 $(CONF_PATH)/$(MYNEWS_CONF)');
mkprinttab('$(CHOWN) $(MN_USER) $(CONF_PATH)/$(MYNEWS_CONF)');
mkprinttab('$(CHGRP) $(MN_GROUP) $(CONF_PATH)/$(MYNEWS_CONF)');
mkprint();
mkprint('uninstall : uninstall_scripts uninstall_conf dbreset');
mkprinttab('$(NOECHO) $(ECHO)');
mkprinttab('$(NOECHO) $(ECHO) MyNews uninstalled successfully');
mkprinttab('$(NOECHO) $(ECHO)');
mkprint();
mkprint('uninstall_scripts :');
mkprinttab('$(RM_F) $(INSTALL_PATH)/bin/$(MYNEWS)');
mkprinttab('$(RM_F) $(INSTALL_PATH)/bin/$(MYDOC)');
mkprinttab('$(RM_F) $(INSTALL_PATH)/bin/$(MYCHECK)');
mkprint();
mkprint('uninstall_conf :');
mkprinttab('$(RM_F) $(CONF_PATH)/$(MYNEWS_CONF)');
mkprint();
mkprint('test : $(BUILD_DIR) $(BUILD_DIR)/$(MYNEWS_CONF)');
mkprinttab('$(NOECHO) $(ECHON) "Testing database version: "');
mkprinttab('$(NOECHO) $(PERL) $(MYNEWS) -c $(BUILD_DIR)/$(MYNEWS_CONF) dbversion');
mkprinttab('$(PERL) $(MYNEWS) -c $(BUILD_DIR)/$(MYNEWS_CONF) dbinit db/test.$(DB_TYPE)');
mkprinttab('$(NOECHO) $(ECHO)');
mkprinttab('$(NOECHO) $(ECHO) MyNews tested successfully');
mkprinttab('$(NOECHO) $(ECHO)');

close(MAKE);
